using System;
using System.Drawing;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Globalization;

namespace CustomControlFreak
{
    public enum eFormatType { DDMMYYYY, MMDDYYYY };
    public enum eCalendarType { TextBoxAndImage, JustCalendar, TextboxAndButton, JustImage, JustButton, JustTextBox };

    [ToolboxData("<{0}:DaintyDate runat=server></{0}:DaintyDate>")]
    [ToolboxBitmapAttribute(typeof(DaintyDate), "CustomControlFreak.calendar.bmp")]
    public class DaintyDate : WebControl, IPostBackDataHandler
    {
        #region Properties
        private Color headerBackColor = Color.CornflowerBlue;
        private Color backColor = Color.FromName("#FFFFFF");
        private Color normalDayColor = Color.FromName("#FFFFFF");
        private Color otherMonthDayFontColor = Color.FromName("#B3B3B3");
        private System.Web.UI.WebControls.FontInfo headerFont;
        private bool displayTodayButton = false;
        private bool displayNoneButton = false;
        private bool displayWeekNumber = false;
        private bool enableTextBox = true;
        private bool abbreviateMonths = false;
        private bool showDefaultDate = true;
        private bool validDateSelected = false;
        private eCalendarType calendarType = eCalendarType.TextBoxAndImage;
        private string buttonText = "Select";                       // default button text
        private eFormatType formatType = eFormatType.DDMMYYYY;      // default dateformat
        private DateTime selectedDate = DateTime.Now;               // default selected time
        private DateTime defaultDisplayDate = DateTime.Now;         // default display time
        private string cultureName = CultureInfo.CurrentCulture.Name;   // default culture
        private CultureInfo culture;
        private string CalendarBodyStyle = "";
        private string CSS = "";

        [Bindable(true), Category("Behavior"), DefaultValue(""), Description("The date format of the control")]
        public eFormatType FormatType
        {
            get { return formatType; }
            set { formatType = value; }
        }

        [Bindable(true), Category("Behavior"), DefaultValue(""), Description("The default selected date")]
        public DateTime SelectedDate
        {
            get
            {
                try
                {
                    return (DateTime)ViewState["SelectedDate"];
                }
                catch { return new DateTime(); }

            }
            set
            {
                validDateSelected = true;
                ViewState["SelectedDate"] = defaultDisplayDate = value;
            }
        }

        [Bindable(true), Category("Behavior"), DefaultValue(""), Description("The text in the textbox")]
        public String Text
        {
            get { return (ViewState["txtValue"] != null) ? (string)ViewState["txtValue"] : null; }
            internal set
            {
                try
                {
                    string[] arr = value.Split(new char[] { '/' });

                    if (formatType == eFormatType.DDMMYYYY)
                    {
                        ViewState["SelectedDate"] = new DateTime(Int32.Parse(arr[2]), Int32.Parse(arr[1]), Int32.Parse(arr[0]));
                    }
                    if (formatType == eFormatType.MMDDYYYY)
                    {
                        ViewState["SelectedDate"] = new DateTime(Int32.Parse(arr[2]), Int32.Parse(arr[0]), Int32.Parse(arr[1]));
                    }

                    ViewState["txtValue"] = value;
                    showDefaultDate = false;
                    validDateSelected = true;
                }
                catch
                {
                    //if (ViewState["txtValue"].ToString() == "")
                    //{
                    //    ViewState["SelectedDate"] = new DateTime();
                    //    showDefaultDate = false;
                    //}
                    //else
                    //{
                    //    ViewState["txtValue"] = "";
                    //    showDefaultDate = true;
                    //}

                    showDefaultDate = true;
                    validDateSelected = false;
                }
            }
        }

        [Bindable(false), Category("Behavior"), DefaultValue("")]
        public bool ValidDateSelected
        {
            get { return validDateSelected; }
        }

        [Bindable(true), Category("Appearance"), Description("The font of the header")]
        public System.Web.UI.WebControls.FontInfo HeaderFont
        {
            get { return headerFont; }
            set { headerFont = value; }
        }

        [Bindable(true), Category("Appearance"), Description("The Background color of the header")]
        public Color HeaderBackColor
        {
            get { return headerBackColor; }
            set { headerBackColor = value; }
        }

        [Bindable(true), Category("Appearance"), Description("The Background color of the calendar")]
        public override Color BackColor
        {
            get { return backColor; }
            set { backColor = value; }
        }

        [Bindable(true), Category("Appearance"), Description("The border color of an normal day")]
        public Color NormalDayColor
        {
            get { return normalDayColor; }
            set { normalDayColor = value; }
        }

        [Bindable(true), Category("Appearance"), Description("The Font Color for last month and next months day")]
        public Color OtherMonthDayFontColor
        {
            get { return otherMonthDayFontColor; }
            set { otherMonthDayFontColor = value; }
        }

        [Bindable(true), Category("Behavior"), DefaultValue(false), Description("Display the today button")]
        public bool DisplayTodayButton
        {
            get { return displayTodayButton; }
            //set { displayTodayButton = value;}
        }

        [Bindable(true), Category("Behavior"), DefaultValue(false), Description("Display the Date None button")]
        public bool DisplayNoneButton
        {
            get { return displayNoneButton; }
            //set { displayNoneButton = value; }
        }

        [Bindable(true), Category("Behavior"), DefaultValue(false), Description("Display the wek number to the left of the calendar")]
        public bool DisplayWeekNumber
        {
            get { return displayWeekNumber; }
            set { displayWeekNumber = value; }
        }

        [Bindable(true), Category("Behavior"), DefaultValue(false), Description("The calendar type to display")]
        public eCalendarType CalendarType
        {
            get { return calendarType; }
            set { calendarType = value; }
        }

        [Bindable(true), Category("Appearance"), DefaultValue("Select"), Description("The text to display on the button")]
        public string ButtonText
        {
            get { return buttonText; }
            set { buttonText = value; }
        }

        [Bindable(true), Category("Behavior"), DefaultValue(true), Description("Enable the user to enter their own values")]
        public bool EnableTextBox
        {
            get { return enableTextBox; }
            set { enableTextBox = value; }
        }

        [Bindable(true), Category("Appearance"), DefaultValue(false), Description("Abbreviate the calendar months to the first three letters")]
        public bool AbbreviateMonths
        {
            get { return abbreviateMonths; }
            set { abbreviateMonths = value; }
        }

        [Bindable(true), Category("Behavior"), Description("Culture")]
        public string CultureName
        {
            set
            {
                this.culture = System.Globalization.CultureInfo.GetCultureInfo(value);
                this.cultureName = value;
            }
            get { return this.cultureName; }
        }

        #endregion

        public DaintyDate()
        {
            this.culture = System.Globalization.CultureInfo.GetCultureInfo(cultureName);
            formatType = eFormatType.MMDDYYYY;
            this.headerFont = this.Font;

            if (!DesignMode)
            {

            }
        }

        protected void BuildStyleSheet()
        {   //removed from headerstyle becasue it was empty
            //Font-Family:" + headerFont.Name + @"
            // Font-Size:" + headerFont.Size + @"px;

            CSS = "<style>";
            //Build the CSS blocks
            CSS += @"." + GetInstanceID() + @"HeaderStyle{
                        text-align:center;
                        
                        
                        background-color:" + headerBackColor.Name.ToString() + @";
                        height:18px;
                     }
                    ." + GetInstanceID() + @"day{
                        text-align:center;
                        font-size:11px;
                        width:15px;
                    }

                    ." + GetInstanceID() + @"dim
                    {
	                    background-color:" + BackColor.Name.ToString() + @";
                        text-align:center;
                        font-size:11px;
                        color:" + otherMonthDayFontColor.Name.ToString() + @";
                        width:15px;
                    }

                    ." + GetInstanceID() + @"dayMn{
                        text-align:center;
                        font-size:11px; 
                        width:15px;
                        border-bottom: solid #B3B3B3 1px;
                    }

                    ." + GetInstanceID() + @"header{
                        text-align:center;
                        font-size:12px; 
                    }

                    ." + GetInstanceID() + @"selectedDayCell{
                        border-style:dotted;
                        border-width:1px;
                        border-color:#B35412;
                        background-color:#F3E18D;
                        cursor:pointer;
                    }

                    ." + GetInstanceID() + @"normalDayCell{
                        border-style:solid;
                        border-width:1px;
                        border-color:" + normalDayColor.Name.ToString() + @";
                        background-color:" + backColor.Name.ToString() + @";
                        cursor:pointer;
                        width:15px;
                        height:15px;
                    }

                    ." + GetInstanceID() + @"mainTable{
                        width:160px;height:140px;
                        font-family:Arial;
                        background-color:" + backColor.Name.ToString() + @";
                        border-width:1px;
                        border-style:solid;
                        border-color:#8A99BA;";

            //only hide the mainTable if its not a JustCalendar
            CSS += (this.calendarType != eCalendarType.JustCalendar) ? "visibility:hidden;position:absolute;" : "";

            CSS += @"}

                    ." + GetInstanceID() + @"weekNumber{
                        background-color:#9FBEF4;
                    }
                    
                    ." + GetInstanceID() + @"mOverHighlight{
                        background-color:white;
                        color:black;
                    }

                    ." + GetInstanceID() + @"mOverHighlightHover{
                        background-color:#316AC5;
                        color:white;
                        cursor:pointer;
                    }

                    ." + GetInstanceID() + @"dropMenu{
                        border-color:#8A99BA;
                        z-index:99999;
                        border-style:solid;
                        border-width:1px;
                        font-family:Arial;
                        font-size:10px;
                        background-color:White;
                        visibility:hidden;
                        position:absolute;
                    }

                    ";

            CSS += "</style>";
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            string JSAxdLink = Page.ClientScript.GetWebResourceUrl(this.GetType(), "CustomControlFreak.lib.js");
            string Arr1AxdLink = Page.ClientScript.GetWebResourceUrl(this.GetType(), "CustomControlFreak.Arrback.gif");
            string Arr2AxdLink = Page.ClientScript.GetWebResourceUrl(this.GetType(), "CustomControlFreak.ArrForward.gif");
            string ArrUpAxdLink = Page.ClientScript.GetWebResourceUrl(this.GetType(), "CustomControlFreak.ArrUp.gif");
            string ArrDownAxdLink = Page.ClientScript.GetWebResourceUrl(this.GetType(), "CustomControlFreak.ArrDown.gif");


            string PageContent = "";
            bool ToggleOff = true;
            string textBoxtStyle = "";

            CalendarBodyStyle = "width:160px;height:140px;font-family:Arial;border-color:#8A99BA;border-width:1px;border-style:solid;";

            writer.Write("<table border=0 cellpadding=0><tr><td valign=\"middle\">");

            if (DesignMode && this.CalendarType == eCalendarType.JustCalendar)
            {
                this.BorderStyle = BorderStyle.Solid;
                this.BorderWidth = 1;
                this.BorderColor = Color.Black;
            }
            else
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Type, "text");
                writer.AddAttribute(HtmlTextWriterAttribute.Name, this.UniqueID);

                if (!enableTextBox)
                    writer.AddAttribute(HtmlTextWriterAttribute.ReadOnly, "true");

                if (this.CalendarType == eCalendarType.JustCalendar)
                {
                    textBoxtStyle += "visibility:hidden;display:none;width:0px;";
                    ToggleOff = false;
                }
                else if (this.CalendarType == eCalendarType.JustImage || this.calendarType == eCalendarType.JustButton)
                {
                    textBoxtStyle += "visibility:hidden;display:none;width:0px;";
                }
                else if (this.CalendarType == eCalendarType.JustTextBox)
                {
                    writer.AddAttribute(HtmlTextWriterAttribute.Onclick, GetInstanceID() + "Class.toggleCalendarShow();");
                }

                writer.AddAttribute(HtmlTextWriterAttribute.Id, "txt" + GetInstanceID());

                if (Text != null)
                {
                    writer.AddAttribute(HtmlTextWriterAttribute.Value, Text);
                }

                //set the style
                writer.AddAttribute(HtmlTextWriterAttribute.Style, textBoxtStyle);
                writer.RenderBeginTag(HtmlTextWriterTag.Input);
                writer.RenderEndTag();
            }

            writer.Write(RendereCalendarType());

            CalendarBodyStyle += (calendarType == eCalendarType.JustCalendar) ? "position:relative;" : "z-index:99998;position:absolute;background-color:" + BackColor.Name.ToString() + "; ";
            CalendarBodyStyle += (!ToggleOff) ? "visibility:visible;" : "visibility:hidden;";

            if (!DesignMode || this.CalendarType == eCalendarType.JustCalendar)
            {
                PageContent += @"<div class='" + GetInstanceID() + "dropMenu' id='divMonths" + GetInstanceID() + "' style='width:70px;'></div>";
                PageContent += @"<div class='" + GetInstanceID() + "dropMenu' id='divYears" + GetInstanceID() + "' style='width:52px;'></div>";

                PageContent += @"<script src='" + JSAxdLink + @"'></script>
                <table class='" + GetInstanceID() + @"mainTable' id='calendar" + GetInstanceID() + "' cellpadding='7' onclick='" + GetInstanceID() + @"Class.keepCalendarOpen();'>
                <tr><td align='center' valign='top'><table border='0' width='150' cellpadding='0' cellspacing='0'>
                <tr class='" + GetInstanceID() + @"HeaderStyle'><td style='text-align:right;width:12px;'>
                <img src='" + Arr1AxdLink + @"' style='cursor:pointer;' onClick='" + GetInstanceID() + @"Class.monthBack();' />
                </td><td style='width:126px;'><div id='divHeader" + GetInstanceID() + @"'>";

                // Show the current month if in design mode
                PageContent += (DesignMode) ? this.culture.DateTimeFormat.MonthNames[DateTime.Now.Month - 1] : "";

                PageContent += @"</div></td>
                <td style='text-align:left;width:12px;'><img  src='" + Arr2AxdLink + @"' style='cursor:pointer;' onClick='" + GetInstanceID() + @"Class.monthForward();'/>
                </td></tr><tr><td colspan='3' align='center'><div id='divDays" + GetInstanceID() + @"' style='font-size:x-small'></div>";

                //PageContent += (DisplayTodayButton || DisplayNoneButton) ? "<hr>" : "";
                //PageContent += (DisplayTodayButton) ? "<input style='width:50px;font-size:x-small;' type='button' value='None' onclick=\"" + GetInstanceID() + "Class.setNoneDate(); \" />&nbsp;&nbsp;" : "";
                //PageContent += (DisplayNoneButton) ? "<input style='width:50px;font-size:x-small;' type='button' value='Today' onclick=\"" + GetInstanceID() + "Class.setTodayDate();\" />" : "";

                PageContent += @"</td></tr></table></td></tr></table>";

                PageContent += @"<input type='hidden' id='" + GetInstanceID() + @"_dateValue'/>
                <script>var months = new Array(''," + this.GetMonths() + @");var days = new Array(" + this.GetDaysOfTheWeek() + @");
                var daysInMonth = new Array(0,31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
                var divDays = document.getElementById('divDays" + GetInstanceID() + @"');
                var divHeader = document.getElementById('divHeader" + GetInstanceID() + @"');
                var upImage = '" + ArrUpAxdLink + @"';
                var downImage = '" + ArrDownAxdLink + @"';
                var " + GetInstanceID() + @"Class = new DateControl(divDays, divHeader, '" + GetInstanceID() + @"'," + DisplayWeekNumber.ToString().ToLower() + @", '" + defaultDisplayDate.Year + @"','" + defaultDisplayDate.Month + @"','" + defaultDisplayDate.Day + @"', " + showDefaultDate.ToString().ToLower() + ", '" + formatType + "'," + ToggleOff.ToString().ToLower() + @",upImage,downImage);
                " + GetInstanceID() + @"Class.renderDays();</script>";

            }




            BuildStyleSheet();
            writer.Write(CSS);
            writer.Write(PageContent);
        }

        protected string RendereCalendarType()
        {
            string render = "</td><td valign=\"middle\">";
            string IconAxdLink = Page.ClientScript.GetWebResourceUrl(this.GetType(), "CustomControlFreak.CalendarIcon.gif");

            Page.ClientScript.RegisterClientScriptBlock(typeof(string), "preloadImages", @"
                                            var cacheThis = new Image();
                                            button.src = '" + IconAxdLink + @"';", true);

            if (this.CalendarType == eCalendarType.TextBoxAndImage)
            {
                render += "<img src='" + IconAxdLink + "' style='Cursor:pointer;' onClick=\"" + GetInstanceID() + "Class.toggleCalendarShow()\"></td>";
            }
            else if (this.CalendarType == eCalendarType.TextboxAndButton)
            {
                render += "<input type='button' value='" + buttonText + "' onClick=\"" + GetInstanceID() + "Class.toggleCalendarShow()\"></td>";

            }
            else if (this.CalendarType == eCalendarType.JustImage)
            {
                render += "<img src='" + IconAxdLink + "' style='Cursor:pointer;' onClick=\"" + GetInstanceID() + "Class.toggleCalendarShow()\"></td>";
            }
            else if (this.CalendarType == eCalendarType.JustButton)
            {
                render += "<input type='button' value='" + buttonText + "' onClick=\"" + GetInstanceID() + "Class.toggleCalendarShow()\"></td>";
            }

            render += "</td></tr></table>";

            return render;
        }

        #region Days and months cultrue
        protected string GetMonths()
        {
            string months = "";
            string month = "";

            for (int i = 0; i < 12; i++)
            {

                if (abbreviateMonths)
                    month = this.culture.DateTimeFormat.AbbreviatedMonthNames[i];
                else
                    month = this.culture.DateTimeFormat.MonthNames[i];

                months += "\"" + month + "\"";

                if (i != 11)
                    months += ",";
            }

            return months;
        }

        protected string GetDaysOfTheWeek()
        {
            string days = "";

            for (int i = 0; i < 7; i++)
            {

                days += "\"" + this.culture.DateTimeFormat.AbbreviatedDayNames[i][0].ToString().ToUpper() + "\"";

                if (i != 6)
                    days += ",";
            }

            return days;
        }
        #endregion

        #region Postback handling

        public virtual bool LoadPostData(string PostDataKey, NameValueCollection postCollection)
        {
            Text = postCollection[PostDataKey];
            return false;
        }

        public virtual void RaisePostDataChangedEvent()
        {
            //empty
        }

        #endregion

        private string GetInstanceID()
        {
            return this.UniqueID.Replace("$", "a");
        }
    }
}